#!/bin/bash
#
#   Hitachi Fibre Channel Adapter Driver Installer for Linux IA32/IA64
#
#   $Header: /home/cvs_linux/hfclinux/hfcinst.sh,v 1.3 2005/04/13 13:23:28 eteshima Exp $
#   $Log: hfcinst.sh,v $
#   Revision 1.3  2005/04/13 13:23:28  eteshima
#   check rmmod return value
#
#   Revision 1.2  2005/04/13 13:00:45  eteshima
#   add header
#
#   Revision 1.3   2005/02/03  IA64 support
#   Revision 1.2   2004/12/15  Check whether module is loaded at the issuing insmod cmd
#   Revision 1.1   2004/12/14  New release
#
# $1: install   -> install
#     uninstall -> uninstall
# $2: Kernel Version  ex. 2.4.21-4EL
#   : NULL -> Kernel Version = `uname -r`
#
if [ "$1" != "install" -a "$1" != "uninstall" ]; then
        echo "Usage : hfcinst.sh install/uninstall [Kernel-Version]"
        echo "example(install  ) : hfcinst.sh install"
        echo "       (uninstall) : hfcinst.sh uninstall"
        exit 1
fi
DRVDIR="drivers"
DRVNAME="hfcldd.o"
DRVMODU="hfcldd"
HFCMKNOD="hfcmknod"
HOSTTYPE=$(uname -m)
TOOLSDIR="/opt/hitachi/drivers/hba"
if [ "$HOSTTYPE" = "ia64" ]; then
	BOOTDIR="boot/efi/efi/redhat"
else
	BOOTDIR="boot"
fi
echo " boot directory is $BOOTDIR"
if [ -n "$2" ]; then
    KERNVER=$2
else
    KERNVER=`uname -r`
fi
LIBMODULE="/lib/modules/$KERNVER/kernel/drivers"
if [ ! -d "$LIBMODULE" ]; then
    echo " @ERR install directry($LIBMODULE) not found"
    if [ -n "$2" ]; then
        echo "      Kernel $2 not installed."
    fi
    exit 1
fi
echo "      Kernel version      : $KERNVER"
echo "      Host Type           : `uname -m`"
if [ "$1" = "install" ]; then
    echo " ---- Install  @Hitachi Fibre Channel Adapter Driver "
    echo "        install -d -o root -g root $LIBMODULE/scsi/"
    install -d -o root -g root $LIBMODULE/scsi/
    echo "        install -o root -g root $DRVDIR/$KERNVER/$DRVNAME $LIBMODULE/scsi/" 
    install -o root -g root $DRVDIR/$KERNVER/$DRVNAME $LIBMODULE/scsi/
    if [ ! -f "$LIBMODULE/scsi/$DRVNAME" ]; then
        echo " @ERR install command failed."
        exit 1
    fi
    echo "      Modify /etc/modules.conf; old file is /etc/modules.conf.backup"
    cp /etc/modules.conf /etc/modules.conf.backup
    if [ `grep -c "scsi_hostadapter[0-9]*[[:space:]]\+hfcldd" /etc/modules.conf` -eq 0 ]; then
        COUNT=`grep -c "scsi_hostadapter" /etc/modules.conf`
        if [ $COUNT -eq 0 ]; then
            echo "alias scsi_hostadapter hfcldd" >> /etc/modules.conf
        elif [ $COUNT -eq 1 ]; then
            if [ `grep -c "scsi_hostadapter0" /etc/modules.conf` -eq 0 ]; then
                LN=`sed -n /scsi_hostadapter/{=} /etc/modules.conf`
                sed s/scsi_hostadapter/scsi_hostadapter0/ /etc/modules.conf > /etc/modules.conf.temp
                mv /etc/modules.conf.temp /etc/modules.conf
                LN=`expr $LN + 1`
                sed "$LN"i"alias scsi_hostadapter$COUNT hfcldd" /etc/modules.conf > /etc/modules.conf.temp
                mv /etc/modules.conf.temp /etc/modules.conf
            else
                LN=`sed -n /scsi_hostadapter0/{=} /etc/modules.conf`
                LN=`expr $LN + 1`
                sed "$LN"i"alias scsi_hostadapter$COUNT hfcldd" /etc/modules.conf > /etc/modules.conf.temp
                mv /etc/modules.conf.temp /etc/modules.conf
            fi
        else
            COUNT=`expr $COUNT - 1`
            LN=`sed -n /scsi_hostadapter$COUNT/{=} /etc/modules.conf`
            LN=`expr $LN + 1`
            COUNT=`expr $COUNT + 1`
            sed "$LN"i"alias scsi_hostadapter$COUNT hfcldd" /etc/modules.conf > /etc/modules.conf.temp
            mv /etc/modules.conf.temp /etc/modules.conf
        fi
    fi
    if [ `grep -c "max_scsi_luns" /etc/modules.conf` -eq 0 ]; then
        echo "options scsi_mod max_scsi_luns=128" >> /etc/modules.conf
    fi
    cp /$BOOTDIR/initrd-$KERNVER.img /$BOOTDIR/initrd-$KERNVER.img.backup
    echo "      Update boot image file /$BOOTDIR/initrd-$KERNVER.img"
    echo "      old boot image file --> /$BOOTDIR/initrd-$KERNVER.img.backup"
    echo "        /sbin/mkinitrd -f /$BOOTDIR/initrd-$KERNVER.img $KERNVER"
    /sbin/mkinitrd -f /$BOOTDIR/initrd-$KERNVER.img $KERNVER
    if [ $? -ne 0 ]; then
        echo " @ERR mkinitrd command fail"
        cp /$BOOTDIR/initrd-$KERNVER.img.backup /$BOOTDIR/initrd-$KERNVER.img
        rm -f /$BOOTDIR/initrd-$KERNVER.img.backup
        exit 1
    fi 
    if [ "$KERNVER" = `uname -r` ]; then
        echo "      check loaded modules"
        if [ `/sbin/lsmod | grep hfcldd | wc -l` -eq 0 ]; then
            echo "        /sbin/insmod $DRVMODU"
            /sbin/insmod $DRVMODU 
            if [ $? -eq 1 ]; then
                echo " @ERR insmod command fail"
                exit 1
            fi
        else
            echo "      A module named hfcldd already exists"
            echo "      Skip insmod. Need reload a module again or reboot to update"
        fi
        echo "      Make special file"
        ./hfcmknod 
        if [ $? -ne 0 ]; then
            exit 1
        fi      
    fi
    install -d -o root -g root $TOOLSDIR     
    install -o root -g root tools/* $TOOLSDIR
    install -o root -g root $HFCMKNOD /etc/init.d/
    /sbin/chkconfig --add hfcmknod
    if [ -f "hbaapi/libhfchbaapi.so" ]; then
        install -o root -g root hbaapi/libhfchbaapi.so /usr/lib/
        echo " ---- Install HBAAPI "
        if [ ! -f "/etc/hba.conf" ]; then
	    echo "hfcldd /usr/lib/libhfchbaapi.so" > /etc/hba.conf
        else
	    COUNT=`grep -c "/usr/lib/libhfchbaapi" /etc/hba.conf`
	    if [ $COUNT -eq 0 ]; then
	        cp /etc/hba.conf /etc/hba.conf.backup
	        echo "hfcldd /usr/lib/libhfchbaapi.so" >> /etc/hba.conf
	    fi
        fi
    fi
    echo " ---- Install Success "
else
    echo " ---- Uninstall  @Hitachi Fibre Channel Adapter Driver "
    echo "        rm -f $LIBMODULE/scsi/$DRVNAME" 
    rm -f $LIBMODULE/scsi/$DRVNAME
    echo "      Modify /etc/modules.conf"
    cp /etc/modules.conf /etc/modules.conf.temp
    sed '/alias scsi_hostadapter.* hfcldd/d' /etc/modules.conf.temp > /etc/modules.conf
    rm /etc/modules.conf.temp
    echo "        /sbin/mkinitrd -f /$BOOTDIR/initrd-$KERNVER.img $KERNVER"
    cp /$BOOTDIR/initrd-$KERNVER.img /$BOOTDIR/initrd-$KERNVER.img.backup
    /sbin/mkinitrd -f /$BOOTDIR/initrd-$KERNVER.img $KERNVER
    if [ $? -ne 0 ]; then
        echo " @ERR mkinitrd command fail"
        cp /$BOOTDIR/initrd-$KERNVER.img.backup /$BOOTDIR/initrd-$KERNVER.img
        exit 1
    fi 
    if [ "$KERNVER" = `uname -r` ]; then
        echo "        /sbin/rmmod $DRVMODU"
        /sbin/rmmod $DRVMODU 
        if [ $? -eq 1 ]; then
           echo " @ERR rmmod command fail"
           exit 1
        fi
        /etc/init.d/hfcmknod stop 
        if [ $? -ne 0 ]; then
            exit 1
        fi      
    fi 
    /sbin/chkconfig --del hfcmknod
    rm -f /etc/init.d/$HFCMKNOD
    rm -f $TOOLSDIR/*
    if [ -f "/etc/hba.conf" ]; then
        COUNT=`grep -c "libhfchbaapi" /etc/hba.conf`
        if [ $COUNT -eq 1 ]; then
            echo " ---- Uninstall HBAAPI"
            cp /etc/hba.conf /etc/hba.conf.temp
            sed "/libhfchbaapi/d" /etc/hba.conf.temp > /etc/hba.conf
            rm /etc/hba.conf.temp
            if [ ! -s /etc/hba.conf ]; then
                rm /etc/hba.conf
            fi
            rm -f /usr/lib/libhfchbaapi.so
        fi
    fi
    echo " ---- Uninstall Success "
fi
